<?php
// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Admin Class
 *
 * Manage Admin Panel Class
 *
 * @package WooCommerce - Points and Rewards
 * @since 1.0.0
 */
class Woo_Pr_License {

	/**
	 * Handles the license activation & deactivation action
	 *
	 * @package WooCommerce - Points and Rewards
	 * @since 1.0.0
	 */
	public function woo_pr_activate_license() {

		$license_key    = sanitize_text_field( $_POST['license_key'] );
		$email          = sanitize_email( $_POST['email'] );
		$license_action = sanitize_text_field( $_POST['license_action'] );

		if ( $license_action == 'Activate License' ) {
			$data = $this->woo_pr_activation_settings( $license_key, $email, $license_action );
			if ( isset( $data['status'] ) && true == $data['status'] ) {
				update_option( 'woo_pr_activation_code', $license_key, false );
				update_option( 'woo_pr_email_address', $email, false );
				$final_activation_code = base64_encode( $license_key . '%' . $email );
				update_option( 'woo_pr_activated', $final_activation_code, false );
			}
			wp_send_json( $data );
		}

		if ( $license_action == 'Deactivate License' ) {
			$license_key = get_option( 'woo_pr_activation_code' );
			$data        = $this->woo_pr_activation_settings( $license_key, $email, $license_action );
			if ( true == $data['status'] ) {
				delete_option( 'woo_pr_activated' );
				delete_option( 'woo_pr_activation_code' );
				delete_option( 'woo_pr_email_address' );
			}
			wp_send_json( $data );
		}
		exit;
	}

	/**
	 * Handles the license activation & deactivation api call
	 *
	 * @package WooCommerce - Points and Rewards
	 * @since 1.0.0
	 */
	public function woo_pr_activation_settings( $license_key, $email, $license_action ) {

		$activation_code = $license_key;
		$email_address   = $email;
		$url             = WOO_PR_LICENSE_VALIDATOR;
		$curl            = curl_init();
		$fields          = array(
			'email'           => $email_address,
			'site_url'        => get_site_url(),
			'activation_code' => $activation_code,
			'activation'      => $license_action,
			'item_id'         => 21945963,
		);

		$fields_string = http_build_query( $fields );
		curl_setopt( $curl, CURLOPT_URL, $url );
		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $curl, CURLOPT_HEADER, false );
		curl_setopt( $curl, CURLOPT_POST, true );
		curl_setopt( $curl, CURLOPT_POSTFIELDS, $fields_string );		
		$data = json_decode( curl_exec( $curl ), true );
		return( $data );
	}


	/**
	 * Handle to display licence activation notice
	 *
	 * @package WooCommerce - Points and Rewards
	 * @since 1.0.0
	 */
	public function woo_pr_show_license_notice() {

		if ( ! woo_pr_is_license_activated() ) {
			if ( function_exists( 'get_current_screen' ) ) {
				// get the current screen.
				$screen          = get_current_screen();
				$is_license_page = false;
				if ( $screen->id == 'woocommerce_page_wc-settings' ) {
					if ( ( isset( $_GET['tab'] ) && $_GET['tab'] != 'woopr-settings' ) || ! isset( $_GET['tab'] ) ) {
						$is_license_page = true;
					} else {
						$is_license_page = false;
					}
				} else {
					$is_license_page = true;
				}

				if ( $is_license_page ) { ?>
					<div class="notice notice-error is-dismissible">
						<p>
						<?php
						$license_page_url = add_query_arg(
							array(
								'page' => 'wc-settings',
								'tab'  => 'woopr-settings',
							),
							admin_url( 'admin.php' )
						);
						printf( esc_html__( '%1$sWooCommerce Points and Rewards%2$s: Please %3$sactivate%4$s your license in order to use the plugin.', 'woopoints' ), '<b>', '</b>', '<a href="' . $license_page_url . '">', '</a>' );
						?>
						</p>
					</div>
					<?php
				}
			}
		}
	}

	/**
	 * Handle licence Script & Style
	 *
	 * @package WooCommerce - Points and Rewards
	 * @since 1.0.0
	 */
	public function woo_pr_enqueue_license_script( $hook ) {

		if ( 'woocommerce_page_wc-settings' === $hook ) {

			$license_status = woo_pr_is_license_activated();
			if ( ( isset( $_GET['section'] ) && $_GET['section'] == 'pr_license' ) || ! ( $license_status ) ) {
				$is_pr_license = 'pr_license';
			} else {
				$is_pr_license = false;
			}
			wp_register_script( 'woo-pr-license-script', WOO_PR_URL . 'includes/js/woo-pr-license.js', array(), WOO_PR_VERSION, true );
			wp_localize_script(
				'woo-pr-license-script',
				'WOO_PR_LICENSE',
				array(
					'ajaxurl'          => admin_url( 'admin-ajax.php' ),
					'woo_pr_activated' => ( $license_status ) ? 'Deactivate License' : 'Activate License',
					'is_pr_license'    => $is_pr_license,
					'is_pr_activated'  => ( $license_status ) ? true : false,
				)
			);
			wp_enqueue_script( 'woo-pr-license-script' );

			wp_enqueue_script( 'sweetalert-script', WOO_PR_URL . 'includes/js/sweetalert2.all.min.js', array( 'jquery' ), WOO_PR_VERSION );

			wp_register_style( 'woo-pr-license-style', WOO_PR_URL . 'includes/css/woo-pr-license.css', array(), WOO_PR_VERSION );
			wp_enqueue_style( 'woo-pr-license-style' );

		}
	}

	/**
	 * Handles to add plugin settings in Woocommerce -> Settings
	 *
	 * @package WooCommerce - Points and Rewards
	 * @since 1.0.0
	 */
	public function woo_pr_admin_settings_tab_license( $settings ) {

		// Add settings in array
		$settings[] = include WOO_PR_ADMIN_DIR . '/class-woo-pr-admin-settings-tabs.php';

		return $settings; // Return.
	}

	/**
	 * Adding Hooks
	 *
	 * @package WooCommerce - Points and Rewards
	 * @since 1.0.0
	 */
	public function add_hooks() {

		// Action to activate & deactivate licens
		add_action( 'wp_ajax_woo_pr_activate_license', array( $this, 'woo_pr_activate_license' ) );

		// Action to display license notice.
		add_action( 'admin_notices', array( $this, 'woo_pr_show_license_notice' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'woo_pr_enqueue_license_script' ) );

		if ( ! woo_pr_is_license_activated() ) {
			add_filter( 'woocommerce_get_settings_pages', array( $this, 'woo_pr_admin_settings_tab_license' ) );
		}
	}
}
